VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "Form1"
   ClientHeight    =   3165
   ClientLeft      =   60
   ClientTop       =   360
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3165
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'---------------------------------
'Title: DirectX Tutorial
'
'Description:
'
'Author: Jacob Roman
'
'Date: 12/04/2005
'
'Contact: jacob_c_roman@hotmail.com
'---------------------------------

Option Explicit

Private Type RECT_SINGLE

    Left As Single
    Top As Single
    Right As Single
    Bottom As Single

End Type

Private Type Animation_Type
    
    Frame_Size() As RECT
    Number_Of_Frames As Long
    Number_Of_Textures As Long
    Current_Frame As Single
    Frame_Counter As Long
    Speed As Single
    Initial_Frame As Long
    Mode As Long 'Single Shot, Loop, etc.
    Offset() As D3DVECTOR2
    Texture_Number() As Long
    
End Type

Private Type Sprite_Type

    X As Single
    Y As Single
    Animation_State As Animation_Type
    Texture_Path() As String
    Texture_List() As Direct3DTexture8
    Total_Number_Of_Textures As Long
    
End Type


'The 2D (Transformed and Lit) vertex format type.
Private Type TLVERTEX

    X As Single
    Y As Single
    Z As Single
    RHW As Single
    Color As Long
    Specular As Long
    TU As Single
    TV As Single
    
End Type

Private Declare Function QueryPerformanceCounter Lib "Kernel32" (lpPerformanceCount As Currency) As Long
Private Declare Function QueryPerformanceFrequency Lib "Kernel32" (lpPerformanceCount As Currency) As Long

Private Const ANIMATION_MODE_SINGLE_SHOT As Long = 0
Private Const ANIMATION_MODE_LOOP As Long = 1

'Some color depth constants to help make the DX constants more readable.
Private Const COLOR_DEPTH_16_BIT As Long = D3DFMT_R5G6B5
Private Const COLOR_DEPTH_24_BIT As Long = D3DFMT_A8R8G8B8
Private Const COLOR_DEPTH_32_BIT As Long = D3DFMT_X8R8G8B8

'The 2D (Transformed and Lit) vertex format.
Private Const FVF_TLVERTEX As Long = D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR

Private DirectX8 As DirectX8 'The master DirectX object.
Private Direct3D As Direct3D8 'Controls all things 3D.
Private Direct3D_Device As Direct3DDevice8 'Represents the hardware rendering.
Private Direct3DX As D3DX8

Private Window_Width As Long
Private Window_Height As Long

Private Fullscreen_Width As Long
Private Fullscreen_Height As Long
Private Color_Depth As Long

Private Fullscreen_Enabled As Boolean 'Helps determine whether it's fullscreen mode.
Private Running As Boolean 'Helps determine whether the main game loop is running.

Private Vertex_List(3) As TLVERTEX '4 vertices will make a square.

Private Sprite As Sprite_Type

Private Frame As RECT_SINGLE

Private Scalar As D3DVECTOR2

Private Time As Single
Private Milliseconds As Single
Private Ticks_Per_Second As Currency
Private Start_Time As Currency

Private Function Hi_Res_Timer_Initialize() As Boolean

    If QueryPerformanceFrequency(Ticks_Per_Second) = 0 Then

        Hi_Res_Timer_Initialize = False

    Else
    
        QueryPerformanceCounter Start_Time
     
        Hi_Res_Timer_Initialize = True
     
    End If

End Function

Private Function Get_Elapsed_Time_Per_Frame() As Single

    Static Last_Time As Currency

    Static Current_Time As Currency

    QueryPerformanceCounter Current_Time
    
    Get_Elapsed_Time_Per_Frame = ((Current_Time - Last_Time) / Ticks_Per_Second) * 60 '60 Frames Per Second
    
    QueryPerformanceCounter Last_Time

End Function

Private Function Get_Elapsed_Time() As Single
    
    Dim Last_Time As Currency
    
    Dim Current_Time As Currency

    QueryPerformanceCounter Current_Time
    
    Get_Elapsed_Time = (Current_Time - Last_Time) / Ticks_Per_Second
    
    QueryPerformanceCounter Last_Time
    
End Function

Private Sub Window_Setup(Window As Form, Optional ByVal X As Long = -1, Optional ByVal Y As Long = -1, Optional ByVal Width As Long = -1, Optional ByVal Height As Long = -1, Optional Caption As String = " ", Optional Auto_Redraw As Boolean = False, Optional ByVal Back_Color As Long = -1)
    
    'Use -1 for default values and "" for default strings.
    
    With Window
    
        If Caption <> " " Then .Caption = Caption 'Else use current setting. Note: Some
                                                  'people may want "" as the caption.
        .AutoRedraw = Auto_Redraw
        .ScaleMode = 3
        If X <> -1 Then .Left = X 'Else use current setting.
        If Y <> -1 Then .Top = Y 'Else use current setting.
        If Width <> -1 Then .Width = Width * Screen.TwipsPerPixelX 'Else use current setting.
        If Height <> -1 Then .Height = Height * Screen.TwipsPerPixelY 'Else use current setting.
        If Back_Color <> -1 Then .BackColor = Back_Color 'Else use current setting.
        .Show
        .Refresh
        .SetFocus
        
    End With
    
End Sub

'This function will make it much easier to setup the vertices with the info it needs.
Private Function Create_TLVertex(X As Single, Y As Single, Z As Single, RHW As Single, Color As Long, Specular As Long, TU As Single, TV As Single) As TLVERTEX

    Create_TLVertex.X = X
    Create_TLVertex.Y = Y
    Create_TLVertex.Z = Z
    Create_TLVertex.RHW = RHW
    Create_TLVertex.Color = Color
    Create_TLVertex.Specular = Specular
    Create_TLVertex.TU = TU
    Create_TLVertex.TV = TV
    
End Function

Private Function DirectX_Initialize() As Boolean

    On Error GoTo Error_Handler
    
    Dim Display_Mode As D3DDISPLAYMODE 'Display mode desciption.
    Dim Direct3D_Window As D3DPRESENT_PARAMETERS 'Backbuffer and viewport description.
    
    Set DirectX8 = New DirectX8 'Creates the DirectX object.
    Set Direct3D = DirectX8.Direct3DCreate() 'Creates the Direct3D object using the DirectX object.
    Set Direct3DX = New D3DX8
    
    If Fullscreen_Enabled = True Then
    
        'Now that we are working with fullscreen mode, we must set up the
        'screen resolution to switch to, rather than use the default screen
        'resolution.
        
        Display_Mode.Width = Fullscreen_Width
        Display_Mode.Height = Fullscreen_Height
        Display_Mode.Format = Color_Depth
    
        Direct3D_Window.Windowed = False 'The app will be in fullscreen mode.
        Direct3D_Window.BackBufferCount = 1 '1 backbuffer only
        Direct3D_Window.BackBufferWidth = Display_Mode.Width 'Match the backbuffer width with the display width
        Direct3D_Window.BackBufferHeight = Display_Mode.Height 'Match the backbuffer height with the display height
        Direct3D_Window.hDeviceWindow = frmMain.hWnd 'Use frmMain as the device window.
        
    Else
    
        Direct3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, Display_Mode 'Use the current display mode that you
                                                                        'are already on. Incase you are confused, I'm
                                                                        'talking about your current screen resolution. ;)
        
        Direct3D_Window.Windowed = True 'The app will be in windowed mode.
    
    End If
    
    Direct3D_Window.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC 'Refresh when the monitor does.
    Direct3D_Window.BackBufferFormat = Display_Mode.Format 'Sets the format that was retrieved into the backbuffer.
    
    'Creates the rendering device with some useful info, along with the info
    'we've already setup for Direct3D_Window.
    Set Direct3D_Device = Direct3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, Direct3D_Window)
    
    Direct3D_Device.SetVertexShader FVF_TLVERTEX 'Set the type of vertex shading. (Required)
    
    Direct3D_Device.SetRenderState D3DRS_SRCBLEND, D3DBLEND_ONE
    Direct3D_Device.SetRenderState D3DRS_DESTBLEND, D3DBLEND_SRCALPHA
    
    Direct3D_Device.SetRenderState D3DRS_ALPHAREF, 255
    Direct3D_Device.SetRenderState D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL
    
    'These lines are not needed, but it's nice to be able to filter the
    'textures to make them look nicer.
    
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_POINT
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_POINT

    Exit Function
    
Error_Handler:
    
    MsgBox "An error occured while initializing DirectX", vbCritical

    DirectX_Initialize = False


End Function

Private Sub Load_Textures()

    Dim Width As Long
    Dim Height As Long
    Dim Transparency_Color As Long
    Dim Current_Texture As Long
    
    With Sprite
    
        .Total_Number_Of_Textures = 4
        
        ReDim .Texture_Path(.Total_Number_Of_Textures) As String
        ReDim .Texture_List(.Total_Number_Of_Textures) As Direct3DTexture8
        
        .Texture_Path(0) = App.Path & "\Graphics\Ken1.bmp"
        .Texture_Path(1) = App.Path & "\Graphics\Ken2.bmp"
        .Texture_Path(2) = App.Path & "\Graphics\Ken3.bmp"
        .Texture_Path(3) = App.Path & "\Graphics\Ken4.bmp"
    
        Width = 1024
        Height = 1024
        
        Transparency_Color = D3DColorRGBA(0, 0, 0, 255)
        
        For Current_Texture = 0 To .Total_Number_Of_Textures - 1
    
            Set .Texture_List(Current_Texture) = Direct3DX.CreateTextureFromFileEx(Direct3D_Device, _
                                                            .Texture_Path(Current_Texture), _
                                                            Width, Height, _
                                                            0, _
                                                            0, _
                                                            D3DFMT_A8R8G8B8, _
                                                            D3DPOOL_MANAGED, _
                                                            D3DX_FILTER_POINT, _
                                                            D3DX_FILTER_POINT, _
                                                            Transparency_Color, _
                                                            ByVal 0, _
                                                            ByVal 0)
                                                            
        Next Current_Texture

    End With

End Sub

Private Sub Setup_Sprite()

    If Fullscreen_Enabled = False Then
    
        Scalar.X = 1
        Scalar.Y = 1
        
    Else
    
        Scalar.X = Fullscreen_Width / Window_Width
        Scalar.Y = Fullscreen_Height / Window_Height
        
    End If
    
    With Sprite
    
        .X = 50
        .Y = 100
        
        With .Animation_State
        
            .Number_Of_Frames = 6
            .Number_Of_Textures = 4
            .Speed = 18
            .Initial_Frame = 0
            .Mode = ANIMATION_MODE_LOOP
            
            ReDim .Frame_Size(.Number_Of_Textures) As RECT
            ReDim .Offset(.Number_Of_Textures) As D3DVECTOR2
            ReDim .Texture_Number(.Number_Of_Frames) As Long
            
            .Frame_Size(0).Left = 0: .Frame_Size(0).Top = 0: .Frame_Size(0).Right = 60: .Frame_Size(0).Bottom = 92
            .Frame_Size(1).Left = 0: .Frame_Size(1).Top = 0: .Frame_Size(1).Right = 61: .Frame_Size(1).Bottom = 91
            .Frame_Size(2).Left = 0: .Frame_Size(2).Top = 0: .Frame_Size(2).Right = 59: .Frame_Size(2).Bottom = 94
            .Frame_Size(3).Left = 0: .Frame_Size(3).Top = 0: .Frame_Size(3).Right = 56: .Frame_Size(3).Bottom = 95
            
            .Offset(0).X = 0: .Offset(0).Y = 3
            .Offset(1).X = -1: .Offset(1).Y = 4
            .Offset(2).X = 1: .Offset(2).Y = 1
            .Offset(3).X = 2: .Offset(3).Y = 0
            
            .Texture_Number(0) = 0
            .Texture_Number(1) = 1
            .Texture_Number(2) = 0
            .Texture_Number(3) = 2
            .Texture_Number(4) = 3
            .Texture_Number(5) = 2
        
        End With
    
    End With
    
End Sub

Private Sub Animate_Sprite()
    
    Dim X As Single, Y As Single
    Dim Current_Texture As Long
    Dim Transparency_Color As Long
    
    'Right here will alphablend the polygon
    Direct3D_Device.SetRenderState D3DRS_ALPHABLENDENABLE, False

    'Right here will give the polygon transparency
    Direct3D_Device.SetRenderState D3DRS_ALPHATESTENABLE, True
        
    With Sprite
    
        X = .X
        Y = .Y
        
        With .Animation_State
    
            If .Number_Of_Frames <> 0 Then
    
                Select Case .Mode
            
                    Case ANIMATION_MODE_SINGLE_SHOT
            
                        If Int(.Current_Frame) > (.Number_Of_Frames - 1) Then
                
                            .Current_Frame = (.Number_Of_Frames - 1)
                
                        End If
                
                    Case ANIMATION_MODE_LOOP
                    
                        If Int(.Current_Frame) > (.Number_Of_Frames - 1) Then
                        
                            .Current_Frame = 0
                            Milliseconds = Get_Elapsed_Time
                        
                        End If
                
                End Select
            
            End If
            
            Current_Texture = .Texture_Number(Int(.Current_Frame))
            
            Transparency_Color = D3DColorRGBA(255, 255, 255, 255)
    
            Frame.Left = (X + .Frame_Size(Current_Texture).Left + .Offset(Current_Texture).X) * Scalar.X
            Frame.Top = (Y + .Frame_Size(Current_Texture).Top + .Offset(Current_Texture).Y) * Scalar.Y
            Frame.Right = (X + .Frame_Size(Current_Texture).Right + .Offset(Current_Texture).X) * Scalar.X
            Frame.Bottom = (Y + .Frame_Size(Current_Texture).Bottom + .Offset(Current_Texture).Y) * Scalar.Y
    
            'Creates the polygon.
            '---------------------------------------------------------------
            With Frame
            
                Vertex_List(0) = Create_TLVertex(.Left, .Top, 0, 1, Transparency_Color, 0, 0, 0)
                Vertex_List(1) = Create_TLVertex(.Right, .Top, 0, 1, Transparency_Color, 0, 1, 0)
                Vertex_List(2) = Create_TLVertex(.Left, .Bottom, 0, 1, Transparency_Color, 0, 0, 1)
                Vertex_List(3) = Create_TLVertex(.Right, .Bottom, 0, 1, Transparency_Color, 0, 1, 1)
            
            End With
            '---------------------------------------------------------------

                                             
            'Set the texture
            Direct3D_Device.SetTexture 0, Sprite.Texture_List(Current_Texture)
            
            .Current_Frame = .Initial_Frame + .Speed * Time
            
        End With
    
    End With
    
End Sub

Private Sub Game_Loop()

    Do While Running = True
        
        DoEvents 'Allow events to happen so the program doesn't lock up.
        
        Time = Get_Elapsed_Time - Milliseconds
        
        '----------------------------------------------------
        'DirectX automatically handles the framerate for you
        'which makes it run (at most) as fast as the monitors
        'refresh rate, so you don't need to add extra code to
        'slow down the loop and run at a certain number of frames
        'per second.
        '----------------------------------------------------
        
        'Clears the backbuffer.
        Direct3D_Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, D3DColorRGBA(0, 0, 0, 0), 1#, 0
            
            Direct3D_Device.BeginScene
                
                Animate_Sprite
            
                'Draw the polygon
                Direct3D_Device.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, Vertex_List(0), Len(Vertex_List(0))
            
            Direct3D_Device.EndScene
        
        'Flips the backbuffer into the form window.
        Direct3D_Device.Present ByVal 0, ByVal 0, 0, ByVal 0
        
        QueryPerformanceCounter Start_Time
        
    Loop

End Sub

Private Sub Close_Program()

    Dim Current_Texture As Long
    
    Running = False 'This helps the program bail out of the game loop.
    
    'Unload all of the DirectX objects.
    
    For Current_Texture = 0 To Sprite.Total_Number_Of_Textures - 1
        
        Set Sprite.Texture_List(Current_Texture) = Nothing
    
    Next Current_Texture
    
    Set Direct3D_Device = Nothing
    Set Direct3DX = Nothing
    Set Direct3D = Nothing
    Set DirectX8 = Nothing
    
    Unload Me 'Unload the form.
    
    End 'Ends the program.
    
    'Although the Unload statement located above exits the program, you
    'will end up with an Automation error after doing so. The End statement
    'will help prevent that, and end the app completely.

End Sub

Private Sub Form_Activate()

    Window_Width = 392
    Window_Height = 251
    
    Fullscreen_Width = 800
    Fullscreen_Height = 600
    Color_Depth = COLOR_DEPTH_16_BIT

    Window_Setup frmMain, -1, -1, Window_Width, Window_Height, "DirectX Tutorial", , RGB(0, 0, 0)
    
    DirectX_Initialize 'Initializes DirectX and Direct3D.
    
    Setup_Sprite
    
    Load_Textures 'Loads textures from files.
    
    Hi_Res_Timer_Initialize
    
    Milliseconds = Get_Elapsed_Time
    
    Running = True 'Initializations all set. It's now ok to activate the game loop.
    
    Game_Loop

End Sub

Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)

    If KeyCode = vbKeyEscape Then 'If the user presses the Esc key...
    
        Close_Program
    
    End If

End Sub

Private Sub Form_Load()

    'This event will fire before the form has completely loaded.
    
    If MsgBox("Click Yes to go to full screen (Recommended)", vbQuestion Or vbYesNo, "Options") = vbYes Then Fullscreen_Enabled = True

End Sub

Private Sub Form_Unload(Cancel As Integer)

    Close_Program

End Sub
